@@ -222,7 +222,7 @@ static void ntfs_device_win32_init_imports(void)
  *
  * Supported flags are O_RDONLY, O_WRONLY and O_RDWR.
  */
-static __inline__ int ntfs_device_unix_status_flags_to_win32(int flags)
+static int ntfs_device_unix_status_flags_to_win32(int flags)
 {
 	int win_mode;
 
@@ -508,7 +508,7 @@ static int ntfs_device_win32_getgeo(HANDLE handle, win32_fd *fd)
  * Return 0 if o.k.
  *	 -1 if not, and errno set.
  */
-static __inline__ int ntfs_device_win32_open_file(char *filename, win32_fd *fd,
+static int ntfs_device_win32_open_file(char *filename, win32_fd *fd,
 		int flags)
 {
 	HANDLE handle;
@@ -538,7 +538,7 @@ static __inline__ int ntfs_device_win32_open_file(char *filename, win32_fd *fd,
  * return 0 if o.k.
  *        -1 if not, and errno set.
  */
-static __inline__ int ntfs_device_win32_open_drive(int drive_id, win32_fd *fd,
+static int ntfs_device_win32_open_drive(int drive_id, win32_fd *fd,
 		int flags)
 {
 	HANDLE handle;
@@ -1113,11 +1113,13 @@ static int ntfs_device_win32_close(struct ntfs_device *dev)
 	}
 	rvl = CloseHandle(fd->handle);
 	free(fd);
+	fd = NULL;
 	if (!rvl) {
 		errno = ntfs_w32error_to_errno(GetLastError());
 		ntfs_log_trace("CloseHandle() failed.\n");
 		return -1;
 	}
+	NDevClearOpen(dev);
 	return 0;
 }
 
@@ -1358,7 +1360,7 @@ static int ntfs_device_win32_stat(struct ntfs_device *dev, struct stat *buf)
  * Return 0 if o.k.
  *	 -1 if not, and errno set.  Note if error fd->handle is trashed.
  */
-static __inline__ int ntfs_win32_hdio_getgeo(struct ntfs_device *dev,
+static int ntfs_win32_hdio_getgeo(struct ntfs_device *dev,
 		struct hd_geometry *argp)
 {
 	win32_fd *fd = (win32_fd *)dev->d_private;
@@ -1380,7 +1382,7 @@ static __inline__ int ntfs_win32_hdio_getgeo(struct ntfs_device *dev,
  * Return 0 if o.k.
  *	 -1 if not, and errno set.  Note if error fd->handle is trashed.
  */
-static __inline__ int ntfs_win32_blksszget(struct ntfs_device *dev,int *argp)
+static int ntfs_win32_blksszget(struct ntfs_device *dev,int *argp)
 {
 	win32_fd *fd = (win32_fd *)dev->d_private;
 	DWORD bytesReturned;
@@ -1447,26 +1449,12 @@ static int ntfs_device_win32_ioctl(struct ntfs_device *dev, int request,
 	}
 }
 
-static s64 ntfs_device_win32_pread(struct ntfs_device *dev, void *b,
-		s64 count, s64 offset)
-{
-	return ntfs_pread(dev, offset, count, b);
-}
-
-static s64 ntfs_device_win32_pwrite(struct ntfs_device *dev, const void *b,
-		s64 count, s64 offset)
-{
-	return ntfs_pwrite(dev, offset, count, b);
-}
-
 struct ntfs_device_operations ntfs_device_win32_io_ops = {
 	.open		= ntfs_device_win32_open,
 	.close		= ntfs_device_win32_close,
 	.seek		= ntfs_device_win32_seek,
 	.read		= ntfs_device_win32_read,
 	.write		= ntfs_device_win32_write,
-	.pread		= ntfs_device_win32_pread,
-	.pwrite		= ntfs_device_win32_pwrite,
 	.sync		= ntfs_device_win32_sync,
 	.stat		= ntfs_device_win32_stat,
 	.ioctl		= ntfs_device_win32_ioctl
