@@ -1340,13 +1340,15 @@ int main(int argc, char *argv[])
 		print_summary();
 
 	if (opts.info || opts.noaction)
-		flags = MS_RDONLY;
+		flags = NTFS_MNT_RDONLY;
+	if (opts.force)
+		flags |= NTFS_MNT_FORCE;
 
-	vol = utils_mount_volume(opts.device, flags, opts.force);
+	vol = utils_mount_volume(opts.device, flags);
 	if (!vol)
 		goto free;
 
-	if ((vol->flags & VOLUME_IS_DIRTY) && (!opts.force))
+	if (NVolWasDirty(vol) && !opts.force)
 		goto umount;
 
 	if (opts.info) {
